{"nbformat":4,"nbformat_minor":0,"metadata":{"colab":{"provenance":[]},"kernelspec":{"name":"python3","display_name":"Python 3"}},"cells":[{"cell_type":"markdown","metadata":{"id":"ZxykTtBbN3dd"},"source":["# Predicting Rental Prices\n","\n","### Machine Learning Practical Session\n","#### Business Analytics Taster Day 2021\n","Some of you are maybe thinking about renting a house in Amsterdam. However, one will be confronted with long waitlists for student housing due to the high demand. So, one may decide to rent privately, becoming vulnerable to overpriced contracts. Deciding if an apartment is worth the ask-price is never easy. Therefore, we will be analyzing house sale data, to gain insights into what the price is determined by. Can we even train a machine learning model to find patterns in the data and predict an accurate price?\n","\n","In this practical, we will go through an entire Data Science. From the basics of data exploration and cleaning up and until using machine learning to train a model to predict house rental prices. The analysis is based on a dataset of Dean De Cock describing the sale of individual residential property in Ames, Iowa, USA from 2006 to 2010."]},{"cell_type":"markdown","metadata":{"id":"2JHXkSQPTSfV"},"source":["#### Coding\n","One does not need to know anything about coding to follow this practical. However, we will show the code and you may follow some of it. We will use the coding language Python, as that language is mostly used in Business for Machine learning. It is important to follow this file chronically and run every piece of code, to make sure everything is initialized correctly. \n","\n","First, it is interesting to try to read the code. For that you need to know some conventions. The following code stores the value 7 into a variable we name *x*. After that, we can use *x* to do a calculation. You can run the code by clicking hovering over the cell and clicking the play/run button or clicking on the code and *Ctrl+Enter*."]},{"cell_type":"code","metadata":{"id":"1duw32_mUcWp","colab":{"base_uri":"https://localhost:8080/"},"executionInfo":{"status":"ok","timestamp":1615816811974,"user_tz":-60,"elapsed":869,"user":{"displayName":"Julia Hazenberg","photoUrl":"","userId":"00349256480520959429"}},"outputId":"e85ca24c-0e00-4daf-abba-a68be21c87fa"},"source":["x = 7\n","x * 6"],"execution_count":null,"outputs":[{"output_type":"execute_result","data":{"text/plain":["42"]},"metadata":{"tags":[]},"execution_count":24}]},{"cell_type":"markdown","metadata":{"id":"FVJEj_xQS0JQ"},"source":["We need to import some software libraries from which we are going to use some functionalities. Run the next code."]},{"cell_type":"code","metadata":{"id":"CB5d9H9dSsu9","executionInfo":{"status":"ok","timestamp":1678803173494,"user_tz":-60,"elapsed":3696,"user":{"displayName":"JEL Kasteren, van","userId":"00965111966476073728"}}},"source":["import numpy as np\n","import pandas as pd\n","import matplotlib.pyplot as plt\n","import seaborn as sns\n","from google.colab import files\n","from sklearn.model_selection import train_test_split\n","from sklearn.tree import DecisionTreeRegressor\n","from sklearn.ensemble import RandomForestRegressor\n","from sklearn import metrics\n","import io"],"execution_count":1,"outputs":[]},{"cell_type":"markdown","metadata":{"id":"ZF7NE9wsXO3i"},"source":["Now, download the *Ames_dataset.csv* from our Google Drive folder to the computer. Afterward, by running the following code, one gets a prompt to upload a file, *Choose file*. Choose the *Ames_dataset.csv* CSV file. We will store this file in the variable *data*. (Ensure that the file is named *Ames_dataset.csv*)."]},{"cell_type":"code","metadata":{"id":"tqQ_oWtOS7vm","colab":{"base_uri":"https://localhost:8080/","height":38},"executionInfo":{"status":"ok","timestamp":1678803186900,"user_tz":-60,"elapsed":13426,"user":{"displayName":"JEL Kasteren, van","userId":"00965111966476073728"}},"outputId":"2b3c9429-fb56-4e8b-9263-abdf10cb4f10"},"source":["uploaded = files.upload()\n"],"execution_count":2,"outputs":[{"output_type":"display_data","data":{"text/plain":["<IPython.core.display.HTML object>"],"text/html":["\n","     <input type=\"file\" id=\"files-bf468b2f-3e7e-4ac3-8b21-187ed1b16168\" name=\"files[]\" multiple disabled\n","        style=\"border:none\" />\n","     <output id=\"result-bf468b2f-3e7e-4ac3-8b21-187ed1b16168\">\n","      Upload widget is only available when the cell has been executed in the\n","      current browser session. Please rerun this cell to enable.\n","      </output>\n","      <script>// Copyright 2017 Google LLC\n","//\n","// Licensed under the Apache License, Version 2.0 (the \"License\");\n","// you may not use this file except in compliance with the License.\n","// You may obtain a copy of the License at\n","//\n","//      http://www.apache.org/licenses/LICENSE-2.0\n","//\n","// Unless required by applicable law or agreed to in writing, software\n","// distributed under the License is distributed on an \"AS IS\" BASIS,\n","// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n","// See the License for the specific language governing permissions and\n","// limitations under the License.\n","\n","/**\n"," * @fileoverview Helpers for google.colab Python module.\n"," */\n","(function(scope) {\n","function span(text, styleAttributes = {}) {\n","  const element = document.createElement('span');\n","  element.textContent = text;\n","  for (const key of Object.keys(styleAttributes)) {\n","    element.style[key] = styleAttributes[key];\n","  }\n","  return element;\n","}\n","\n","// Max number of bytes which will be uploaded at a time.\n","const MAX_PAYLOAD_SIZE = 100 * 1024;\n","\n","function _uploadFiles(inputId, outputId) {\n","  const steps = uploadFilesStep(inputId, outputId);\n","  const outputElement = document.getElementById(outputId);\n","  // Cache steps on the outputElement to make it available for the next call\n","  // to uploadFilesContinue from Python.\n","  outputElement.steps = steps;\n","\n","  return _uploadFilesContinue(outputId);\n","}\n","\n","// This is roughly an async generator (not supported in the browser yet),\n","// where there are multiple asynchronous steps and the Python side is going\n","// to poll for completion of each step.\n","// This uses a Promise to block the python side on completion of each step,\n","// then passes the result of the previous step as the input to the next step.\n","function _uploadFilesContinue(outputId) {\n","  const outputElement = document.getElementById(outputId);\n","  const steps = outputElement.steps;\n","\n","  const next = steps.next(outputElement.lastPromiseValue);\n","  return Promise.resolve(next.value.promise).then((value) => {\n","    // Cache the last promise value to make it available to the next\n","    // step of the generator.\n","    outputElement.lastPromiseValue = value;\n","    return next.value.response;\n","  });\n","}\n","\n","/**\n"," * Generator function which is called between each async step of the upload\n"," * process.\n"," * @param {string} inputId Element ID of the input file picker element.\n"," * @param {string} outputId Element ID of the output display.\n"," * @return {!Iterable<!Object>} Iterable of next steps.\n"," */\n","function* uploadFilesStep(inputId, outputId) {\n","  const inputElement = document.getElementById(inputId);\n","  inputElement.disabled = false;\n","\n","  const outputElement = document.getElementById(outputId);\n","  outputElement.innerHTML = '';\n","\n","  const pickedPromise = new Promise((resolve) => {\n","    inputElement.addEventListener('change', (e) => {\n","      resolve(e.target.files);\n","    });\n","  });\n","\n","  const cancel = document.createElement('button');\n","  inputElement.parentElement.appendChild(cancel);\n","  cancel.textContent = 'Cancel upload';\n","  const cancelPromise = new Promise((resolve) => {\n","    cancel.onclick = () => {\n","      resolve(null);\n","    };\n","  });\n","\n","  // Wait for the user to pick the files.\n","  const files = yield {\n","    promise: Promise.race([pickedPromise, cancelPromise]),\n","    response: {\n","      action: 'starting',\n","    }\n","  };\n","\n","  cancel.remove();\n","\n","  // Disable the input element since further picks are not allowed.\n","  inputElement.disabled = true;\n","\n","  if (!files) {\n","    return {\n","      response: {\n","        action: 'complete',\n","      }\n","    };\n","  }\n","\n","  for (const file of files) {\n","    const li = document.createElement('li');\n","    li.append(span(file.name, {fontWeight: 'bold'}));\n","    li.append(span(\n","        `(${file.type || 'n/a'}) - ${file.size} bytes, ` +\n","        `last modified: ${\n","            file.lastModifiedDate ? file.lastModifiedDate.toLocaleDateString() :\n","                                    'n/a'} - `));\n","    const percent = span('0% done');\n","    li.appendChild(percent);\n","\n","    outputElement.appendChild(li);\n","\n","    const fileDataPromise = new Promise((resolve) => {\n","      const reader = new FileReader();\n","      reader.onload = (e) => {\n","        resolve(e.target.result);\n","      };\n","      reader.readAsArrayBuffer(file);\n","    });\n","    // Wait for the data to be ready.\n","    let fileData = yield {\n","      promise: fileDataPromise,\n","      response: {\n","        action: 'continue',\n","      }\n","    };\n","\n","    // Use a chunked sending to avoid message size limits. See b/62115660.\n","    let position = 0;\n","    do {\n","      const length = Math.min(fileData.byteLength - position, MAX_PAYLOAD_SIZE);\n","      const chunk = new Uint8Array(fileData, position, length);\n","      position += length;\n","\n","      const base64 = btoa(String.fromCharCode.apply(null, chunk));\n","      yield {\n","        response: {\n","          action: 'append',\n","          file: file.name,\n","          data: base64,\n","        },\n","      };\n","\n","      let percentDone = fileData.byteLength === 0 ?\n","          100 :\n","          Math.round((position / fileData.byteLength) * 100);\n","      percent.textContent = `${percentDone}% done`;\n","\n","    } while (position < fileData.byteLength);\n","  }\n","\n","  // All done.\n","  yield {\n","    response: {\n","      action: 'complete',\n","    }\n","  };\n","}\n","\n","scope.google = scope.google || {};\n","scope.google.colab = scope.google.colab || {};\n","scope.google.colab._files = {\n","  _uploadFiles,\n","  _uploadFilesContinue,\n","};\n","})(self);\n","</script> "]},"metadata":{}}]},{"cell_type":"code","metadata":{"id":"S6PTp51HhHrg"},"source":["data = pd.read_csv(io.BytesIO(uploaded['Ames_dataset.csv']))"],"execution_count":null,"outputs":[]},{"cell_type":"markdown","metadata":{"id":"pnMrSWkiBJBA"},"source":["## Data Exploration\n","Naturally, you want to see the data. Is it successfully uploaded? The next line will request the 10 top rows of the dataset to be shown."]},{"cell_type":"code","metadata":{"id":"6ltDB7s2BIcq","colab":{"base_uri":"https://localhost:8080/","height":379},"executionInfo":{"status":"ok","timestamp":1615816838399,"user_tz":-60,"elapsed":936,"user":{"displayName":"Julia Hazenberg","photoUrl":"","userId":"00349256480520959429"}},"outputId":"a5e69123-b810-4397-f138-b7a8fb9067d3"},"source":["data.head(10)"],"execution_count":null,"outputs":[{"output_type":"execute_result","data":{"text/html":["<div>\n","<style scoped>\n","    .dataframe tbody tr th:only-of-type {\n","        vertical-align: middle;\n","    }\n","\n","    .dataframe tbody tr th {\n","        vertical-align: top;\n","    }\n","\n","    .dataframe thead th {\n","        text-align: right;\n","    }\n","</style>\n","<table border=\"1\" class=\"dataframe\">\n","  <thead>\n","    <tr style=\"text-align: right;\">\n","      <th></th>\n","      <th>SalePrice</th>\n","      <th>GrLivArea</th>\n","      <th>TotalBsmtSF</th>\n","      <th>1stFlrSF</th>\n","      <th>2ndFlrSF</th>\n","      <th>Bedroom</th>\n","      <th>TotRmsAbvGrd</th>\n","      <th>Neighborhood</th>\n","      <th>MSZoning</th>\n","      <th>OverallQual</th>\n","      <th>OverallCond</th>\n","      <th>YearBuilt</th>\n","      <th>YearRemodAdd</th>\n","      <th>MSSubClass</th>\n","      <th>BldgType</th>\n","      <th>HouseStyle</th>\n","      <th>RoofStyle</th>\n","      <th>Exterior1st</th>\n","      <th>Exterior2nd</th>\n","      <th>ExterQual</th>\n","      <th>ExterCond</th>\n","      <th>MasVnrType</th>\n","      <th>MasVnrArea</th>\n","      <th>Foundation</th>\n","      <th>BsmtQual</th>\n","      <th>BsmtCond</th>\n","      <th>BsmtExposure</th>\n","      <th>BsmtFinType1</th>\n","      <th>BsmtFinSF1</th>\n","      <th>BsmtUnfSF</th>\n","      <th>CentralAir</th>\n","      <th>Electrical</th>\n","      <th>HeatingQC</th>\n","      <th>BsmtFullBath</th>\n","      <th>BsmtHalfBath</th>\n","      <th>FullBath</th>\n","      <th>HalfBath</th>\n","      <th>Kitchen</th>\n","      <th>KitchenQual</th>\n","      <th>Fireplaces</th>\n","      <th>FireplaceQu</th>\n","      <th>GarageType</th>\n","      <th>GarageYrBlt</th>\n","      <th>GarageFinish</th>\n","      <th>GarageCars</th>\n","      <th>GarageArea</th>\n","      <th>Fence</th>\n","      <th>WoodDeckSF</th>\n","      <th>OpenPorchSF</th>\n","      <th>EnclosedPorch</th>\n","      <th>ScreenPorch</th>\n","      <th>LotFrontage</th>\n","      <th>LotArea</th>\n","      <th>LotShape</th>\n","      <th>LotConfig</th>\n","      <th>PavedDrive</th>\n","      <th>SaleType</th>\n","    </tr>\n","  </thead>\n","  <tbody>\n","    <tr>\n","      <th>0</th>\n","      <td>208500</td>\n","      <td>1710</td>\n","      <td>856</td>\n","      <td>856</td>\n","      <td>854</td>\n","      <td>3</td>\n","      <td>8</td>\n","      <td>CollgCr</td>\n","      <td>RL</td>\n","      <td>7</td>\n","      <td>5</td>\n","      <td>2003</td>\n","      <td>2003</td>\n","      <td>C60</td>\n","      <td>1Fam</td>\n","      <td>2Story</td>\n","      <td>Gable</td>\n","      <td>VinylSd</td>\n","      <td>VinylSd</td>\n","      <td>Gd</td>\n","      <td>TA</td>\n","      <td>BrkFace</td>\n","      <td>196.0</td>\n","      <td>PConc</td>\n","      <td>Gd</td>\n","      <td>TA</td>\n","      <td>No</td>\n","      <td>GLQ</td>\n","      <td>706</td>\n","      <td>150</td>\n","      <td>Y</td>\n","      <td>SBrkr</td>\n","      <td>Ex</td>\n","      <td>1</td>\n","      <td>0</td>\n","      <td>2</td>\n","      <td>1</td>\n","      <td>1</td>\n","      <td>Gd</td>\n","      <td>0</td>\n","      <td>None</td>\n","      <td>Attchd</td>\n","      <td>2003.0</td>\n","      <td>RFn</td>\n","      <td>2</td>\n","      <td>548</td>\n","      <td>None</td>\n","      <td>0</td>\n","      <td>61</td>\n","      <td>0</td>\n","      <td>0</td>\n","      <td>65.0</td>\n","      <td>8450</td>\n","      <td>Reg</td>\n","      <td>Inside</td>\n","      <td>Y</td>\n","      <td>WD</td>\n","    </tr>\n","    <tr>\n","      <th>1</th>\n","      <td>181500</td>\n","      <td>1262</td>\n","      <td>1262</td>\n","      <td>1262</td>\n","      <td>0</td>\n","      <td>3</td>\n","      <td>6</td>\n","      <td>Veenker</td>\n","      <td>RL</td>\n","      <td>6</td>\n","      <td>8</td>\n","      <td>1976</td>\n","      <td>1976</td>\n","      <td>C20</td>\n","      <td>1Fam</td>\n","      <td>1Story</td>\n","      <td>Gable</td>\n","      <td>MetalSd</td>\n","      <td>MetalSd</td>\n","      <td>TA</td>\n","      <td>TA</td>\n","      <td>None</td>\n","      <td>0.0</td>\n","      <td>CBlock</td>\n","      <td>Gd</td>\n","      <td>TA</td>\n","      <td>Gd</td>\n","      <td>ALQ</td>\n","      <td>978</td>\n","      <td>284</td>\n","      <td>Y</td>\n","      <td>SBrkr</td>\n","      <td>Ex</td>\n","      <td>0</td>\n","      <td>1</td>\n","      <td>2</td>\n","      <td>0</td>\n","      <td>1</td>\n","      <td>TA</td>\n","      <td>1</td>\n","      <td>TA</td>\n","      <td>Attchd</td>\n","      <td>1976.0</td>\n","      <td>RFn</td>\n","      <td>2</td>\n","      <td>460</td>\n","      <td>None</td>\n","      <td>298</td>\n","      <td>0</td>\n","      <td>0</td>\n","      <td>0</td>\n","      <td>80.0</td>\n","      <td>9600</td>\n","      <td>Reg</td>\n","      <td>FR2</td>\n","      <td>Y</td>\n","      <td>WD</td>\n","    </tr>\n","    <tr>\n","      <th>2</th>\n","      <td>223500</td>\n","      <td>1786</td>\n","      <td>920</td>\n","      <td>920</td>\n","      <td>866</td>\n","      <td>3</td>\n","      <td>6</td>\n","      <td>CollgCr</td>\n","      <td>RL</td>\n","      <td>7</td>\n","      <td>5</td>\n","      <td>2001</td>\n","      <td>2002</td>\n","      <td>C60</td>\n","      <td>1Fam</td>\n","      <td>2Story</td>\n","      <td>Gable</td>\n","      <td>VinylSd</td>\n","      <td>VinylSd</td>\n","      <td>Gd</td>\n","      <td>TA</td>\n","      <td>BrkFace</td>\n","      <td>162.0</td>\n","      <td>PConc</td>\n","      <td>Gd</td>\n","      <td>TA</td>\n","      <td>Mn</td>\n","      <td>GLQ</td>\n","      <td>486</td>\n","      <td>434</td>\n","      <td>Y</td>\n","      <td>SBrkr</td>\n","      <td>Ex</td>\n","      <td>1</td>\n","      <td>0</td>\n","      <td>2</td>\n","      <td>1</td>\n","      <td>1</td>\n","      <td>Gd</td>\n","      <td>1</td>\n","      <td>TA</td>\n","      <td>Attchd</td>\n","      <td>2001.0</td>\n","      <td>RFn</td>\n","      <td>2</td>\n","      <td>608</td>\n","      <td>None</td>\n","      <td>0</td>\n","      <td>42</td>\n","      <td>0</td>\n","      <td>0</td>\n","      <td>68.0</td>\n","      <td>11250</td>\n","      <td>IR1</td>\n","      <td>Inside</td>\n","      <td>Y</td>\n","      <td>WD</td>\n","    </tr>\n","    <tr>\n","      <th>3</th>\n","      <td>250000</td>\n","      <td>2198</td>\n","      <td>1145</td>\n","      <td>1145</td>\n","      <td>1053</td>\n","      <td>4</td>\n","      <td>9</td>\n","      <td>NoRidge</td>\n","      <td>RL</td>\n","      <td>8</td>\n","      <td>5</td>\n","      <td>2000</td>\n","      <td>2000</td>\n","      <td>C60</td>\n","      <td>1Fam</td>\n","      <td>2Story</td>\n","      <td>Gable</td>\n","      <td>VinylSd</td>\n","      <td>VinylSd</td>\n","      <td>Gd</td>\n","      <td>TA</td>\n","      <td>BrkFace</td>\n","      <td>350.0</td>\n","      <td>PConc</td>\n","      <td>Gd</td>\n","      <td>TA</td>\n","      <td>Av</td>\n","      <td>GLQ</td>\n","      <td>655</td>\n","      <td>490</td>\n","      <td>Y</td>\n","      <td>SBrkr</td>\n","      <td>Ex</td>\n","      <td>1</td>\n","      <td>0</td>\n","      <td>2</td>\n","      <td>1</td>\n","      <td>1</td>\n","      <td>Gd</td>\n","      <td>1</td>\n","      <td>TA</td>\n","      <td>Attchd</td>\n","      <td>2000.0</td>\n","      <td>RFn</td>\n","      <td>3</td>\n","      <td>836</td>\n","      <td>None</td>\n","      <td>192</td>\n","      <td>84</td>\n","      <td>0</td>\n","      <td>0</td>\n","      <td>84.0</td>\n","      <td>14260</td>\n","      <td>IR1</td>\n","      <td>FR2</td>\n","      <td>Y</td>\n","      <td>WD</td>\n","    </tr>\n","    <tr>\n","      <th>4</th>\n","      <td>143000</td>\n","      <td>1362</td>\n","      <td>796</td>\n","      <td>796</td>\n","      <td>566</td>\n","      <td>1</td>\n","      <td>5</td>\n","      <td>Mitchel</td>\n","      <td>RL</td>\n","      <td>5</td>\n","      <td>5</td>\n","      <td>1993</td>\n","      <td>1995</td>\n","      <td>C50</td>\n","      <td>1Fam</td>\n","      <td>1.5Fin</td>\n","      <td>Gable</td>\n","      <td>VinylSd</td>\n","      <td>VinylSd</td>\n","      <td>TA</td>\n","      <td>TA</td>\n","      <td>None</td>\n","      <td>0.0</td>\n","      <td>Wood</td>\n","      <td>Gd</td>\n","      <td>TA</td>\n","      <td>No</td>\n","      <td>GLQ</td>\n","      <td>732</td>\n","      <td>64</td>\n","      <td>Y</td>\n","      <td>SBrkr</td>\n","      <td>Ex</td>\n","      <td>1</td>\n","      <td>0</td>\n","      <td>1</td>\n","      <td>1</td>\n","      <td>1</td>\n","      <td>TA</td>\n","      <td>0</td>\n","      <td>None</td>\n","      <td>Attchd</td>\n","      <td>1993.0</td>\n","      <td>Unf</td>\n","      <td>2</td>\n","      <td>480</td>\n","      <td>MnPrv</td>\n","      <td>40</td>\n","      <td>30</td>\n","      <td>0</td>\n","      <td>0</td>\n","      <td>85.0</td>\n","      <td>14115</td>\n","      <td>IR1</td>\n","      <td>Inside</td>\n","      <td>Y</td>\n","      <td>WD</td>\n","    </tr>\n","    <tr>\n","      <th>5</th>\n","      <td>307000</td>\n","      <td>1694</td>\n","      <td>1686</td>\n","      <td>1694</td>\n","      <td>0</td>\n","      <td>3</td>\n","      <td>7</td>\n","      <td>Somerst</td>\n","      <td>RL</td>\n","      <td>8</td>\n","      <td>5</td>\n","      <td>2004</td>\n","      <td>2005</td>\n","      <td>C20</td>\n","      <td>1Fam</td>\n","      <td>1Story</td>\n","      <td>Gable</td>\n","      <td>VinylSd</td>\n","      <td>VinylSd</td>\n","      <td>Gd</td>\n","      <td>TA</td>\n","      <td>Stone</td>\n","      <td>186.0</td>\n","      <td>PConc</td>\n","      <td>Ex</td>\n","      <td>TA</td>\n","      <td>Av</td>\n","      <td>GLQ</td>\n","      <td>1369</td>\n","      <td>317</td>\n","      <td>Y</td>\n","      <td>SBrkr</td>\n","      <td>Ex</td>\n","      <td>1</td>\n","      <td>0</td>\n","      <td>2</td>\n","      <td>0</td>\n","      <td>1</td>\n","      <td>Gd</td>\n","      <td>1</td>\n","      <td>Gd</td>\n","      <td>Attchd</td>\n","      <td>2004.0</td>\n","      <td>RFn</td>\n","      <td>2</td>\n","      <td>636</td>\n","      <td>None</td>\n","      <td>255</td>\n","      <td>57</td>\n","      <td>0</td>\n","      <td>0</td>\n","      <td>75.0</td>\n","      <td>10084</td>\n","      <td>Reg</td>\n","      <td>Inside</td>\n","      <td>Y</td>\n","      <td>WD</td>\n","    </tr>\n","    <tr>\n","      <th>6</th>\n","      <td>200000</td>\n","      <td>2090</td>\n","      <td>1107</td>\n","      <td>1107</td>\n","      <td>983</td>\n","      <td>3</td>\n","      <td>7</td>\n","      <td>NWAmes</td>\n","      <td>RL</td>\n","      <td>7</td>\n","      <td>6</td>\n","      <td>1973</td>\n","      <td>1973</td>\n","      <td>C60</td>\n","      <td>1Fam</td>\n","      <td>2Story</td>\n","      <td>Gable</td>\n","      <td>HdBoard</td>\n","      <td>HdBoard</td>\n","      <td>TA</td>\n","      <td>TA</td>\n","      <td>Stone</td>\n","      <td>240.0</td>\n","      <td>CBlock</td>\n","      <td>Gd</td>\n","      <td>TA</td>\n","      <td>Mn</td>\n","      <td>ALQ</td>\n","      <td>859</td>\n","      <td>216</td>\n","      <td>Y</td>\n","      <td>SBrkr</td>\n","      <td>Ex</td>\n","      <td>1</td>\n","      <td>0</td>\n","      <td>2</td>\n","      <td>1</td>\n","      <td>1</td>\n","      <td>TA</td>\n","      <td>2</td>\n","      <td>TA</td>\n","      <td>Attchd</td>\n","      <td>1973.0</td>\n","      <td>RFn</td>\n","      <td>2</td>\n","      <td>484</td>\n","      <td>None</td>\n","      <td>235</td>\n","      <td>204</td>\n","      <td>228</td>\n","      <td>0</td>\n","      <td>NaN</td>\n","      <td>10382</td>\n","      <td>IR1</td>\n","      <td>Corner</td>\n","      <td>Y</td>\n","      <td>WD</td>\n","    </tr>\n","    <tr>\n","      <th>7</th>\n","      <td>118000</td>\n","      <td>1077</td>\n","      <td>991</td>\n","      <td>1077</td>\n","      <td>0</td>\n","      <td>2</td>\n","      <td>5</td>\n","      <td>BrkSide</td>\n","      <td>RL</td>\n","      <td>5</td>\n","      <td>6</td>\n","      <td>1939</td>\n","      <td>1950</td>\n","      <td>C190</td>\n","      <td>2fmCon</td>\n","      <td>1.5Unf</td>\n","      <td>Gable</td>\n","      <td>MetalSd</td>\n","      <td>MetalSd</td>\n","      <td>TA</td>\n","      <td>TA</td>\n","      <td>None</td>\n","      <td>0.0</td>\n","      <td>BrkTil</td>\n","      <td>TA</td>\n","      <td>TA</td>\n","      <td>No</td>\n","      <td>GLQ</td>\n","      <td>851</td>\n","      <td>140</td>\n","      <td>Y</td>\n","      <td>SBrkr</td>\n","      <td>Ex</td>\n","      <td>1</td>\n","      <td>0</td>\n","      <td>1</td>\n","      <td>0</td>\n","      <td>2</td>\n","      <td>TA</td>\n","      <td>2</td>\n","      <td>TA</td>\n","      <td>Attchd</td>\n","      <td>1939.0</td>\n","      <td>RFn</td>\n","      <td>1</td>\n","      <td>205</td>\n","      <td>None</td>\n","      <td>0</td>\n","      <td>4</td>\n","      <td>0</td>\n","      <td>0</td>\n","      <td>50.0</td>\n","      <td>7420</td>\n","      <td>Reg</td>\n","      <td>Corner</td>\n","      <td>Y</td>\n","      <td>WD</td>\n","    </tr>\n","    <tr>\n","      <th>8</th>\n","      <td>129500</td>\n","      <td>1040</td>\n","      <td>1040</td>\n","      <td>1040</td>\n","      <td>0</td>\n","      <td>3</td>\n","      <td>5</td>\n","      <td>Sawyer</td>\n","      <td>RL</td>\n","      <td>5</td>\n","      <td>5</td>\n","      <td>1965</td>\n","      <td>1965</td>\n","      <td>C20</td>\n","      <td>1Fam</td>\n","      <td>1Story</td>\n","      <td>Hip</td>\n","      <td>HdBoard</td>\n","      <td>HdBoard</td>\n","      <td>TA</td>\n","      <td>TA</td>\n","      <td>None</td>\n","      <td>0.0</td>\n","      <td>CBlock</td>\n","      <td>TA</td>\n","      <td>TA</td>\n","      <td>No</td>\n","      <td>Rec</td>\n","      <td>906</td>\n","      <td>134</td>\n","      <td>Y</td>\n","      <td>SBrkr</td>\n","      <td>Ex</td>\n","      <td>1</td>\n","      <td>0</td>\n","      <td>1</td>\n","      <td>0</td>\n","      <td>1</td>\n","      <td>TA</td>\n","      <td>0</td>\n","      <td>None</td>\n","      <td>Detchd</td>\n","      <td>1965.0</td>\n","      <td>Unf</td>\n","      <td>1</td>\n","      <td>384</td>\n","      <td>None</td>\n","      <td>0</td>\n","      <td>0</td>\n","      <td>0</td>\n","      <td>0</td>\n","      <td>70.0</td>\n","      <td>11200</td>\n","      <td>Reg</td>\n","      <td>Inside</td>\n","      <td>Y</td>\n","      <td>WD</td>\n","    </tr>\n","    <tr>\n","      <th>9</th>\n","      <td>144000</td>\n","      <td>912</td>\n","      <td>912</td>\n","      <td>912</td>\n","      <td>0</td>\n","      <td>2</td>\n","      <td>4</td>\n","      <td>Sawyer</td>\n","      <td>RL</td>\n","      <td>5</td>\n","      <td>6</td>\n","      <td>1962</td>\n","      <td>1962</td>\n","      <td>C20</td>\n","      <td>1Fam</td>\n","      <td>1Story</td>\n","      <td>Hip</td>\n","      <td>HdBoard</td>\n","      <td>Plywood</td>\n","      <td>TA</td>\n","      <td>TA</td>\n","      <td>None</td>\n","      <td>0.0</td>\n","      <td>CBlock</td>\n","      <td>TA</td>\n","      <td>TA</td>\n","      <td>No</td>\n","      <td>ALQ</td>\n","      <td>737</td>\n","      <td>175</td>\n","      <td>Y</td>\n","      <td>SBrkr</td>\n","      <td>TA</td>\n","      <td>1</td>\n","      <td>0</td>\n","      <td>1</td>\n","      <td>0</td>\n","      <td>1</td>\n","      <td>TA</td>\n","      <td>0</td>\n","      <td>None</td>\n","      <td>Detchd</td>\n","      <td>1962.0</td>\n","      <td>Unf</td>\n","      <td>1</td>\n","      <td>352</td>\n","      <td>None</td>\n","      <td>140</td>\n","      <td>0</td>\n","      <td>0</td>\n","      <td>176</td>\n","      <td>NaN</td>\n","      <td>12968</td>\n","      <td>IR2</td>\n","      <td>Inside</td>\n","      <td>Y</td>\n","      <td>WD</td>\n","    </tr>\n","  </tbody>\n","</table>\n","</div>"],"text/plain":["   SalePrice  GrLivArea  TotalBsmtSF  ...  LotConfig  PavedDrive  SaleType\n","0     208500       1710          856  ...     Inside           Y        WD\n","1     181500       1262         1262  ...        FR2           Y        WD\n","2     223500       1786          920  ...     Inside           Y        WD\n","3     250000       2198         1145  ...        FR2           Y        WD\n","4     143000       1362          796  ...     Inside           Y        WD\n","5     307000       1694         1686  ...     Inside           Y        WD\n","6     200000       2090         1107  ...     Corner           Y        WD\n","7     118000       1077          991  ...     Corner           Y        WD\n","8     129500       1040         1040  ...     Inside           Y        WD\n","9     144000        912          912  ...     Inside           Y        WD\n","\n","[10 rows x 57 columns]"]},"metadata":{"tags":[]},"execution_count":28}]},{"cell_type":"markdown","metadata":{"id":"d1LedUfTBe6K"},"source":["How many data points (rows) does our data have and how many variables (columns)?"]},{"cell_type":"code","metadata":{"id":"Y-AjMiRZBdf_"},"source":["print('# Data points:',len(data),'\\n', '# variables:', data.shape[1])"],"execution_count":null,"outputs":[]},{"cell_type":"markdown","metadata":{"id":"abS5Pu4I8yud"},"source":["To start our analysis we investigate our target, the Sale price of houses (in US dollars). Run the next piece of code to see its distribution."]},{"cell_type":"code","metadata":{"id":"LZG3e3ULSkX9"},"source":["sns.displot(data['SalePrice'], kde=True);"],"execution_count":null,"outputs":[]},{"cell_type":"markdown","metadata":{"id":"o02flgYEpFSd"},"source":["Next, the data consists of variables explain something about the house. Using these explanatory variables (features) we want to predict the house price. In light of the time, some variables were already removed as they were not informative for the model. One such example is *SaleType* Which is visualized when you run the cell below. First, a table will be generated counting the occurrence of every entity in the variable. Second, two figures will be generated describing the data.\n","\n","The description of the variables can be found in the shared Google Drive.\n","\n","#### Question 1, Feature Selection\n","What is the consequence of not taking such variables into account in your machine learning model? Why would it be a good choice and why not?\n","\n","#### Question 2, Boxplots\n","One of the plots shows boxplots, can you explain what a boxplot is?"]},{"cell_type":"code","metadata":{"id":"jlwSU67033LB"},"source":["name = \"SaleType\"\n","print(data[name].value_counts())\n","fig,ax = plt.subplots(1,2,figsize=(13,5))\n","sns.countplot(x=data[name], ax=ax[0]);\n","sns.boxplot(x=name, y='SalePrice',data=data, ax=ax[1]);"],"execution_count":null,"outputs":[]},{"cell_type":"markdown","metadata":{"id":"IimhVrRS__mp"},"source":["Now we will delete (drop) the variable *SaleType* with the next line of code. Running the code above will now result in an Error, as *SaleType* is removed."]},{"cell_type":"code","metadata":{"id":"PR5f3P2yAAR1"},"source":["data.drop('SaleType', axis=1, inplace=True)"],"execution_count":null,"outputs":[]},{"cell_type":"markdown","metadata":{"id":"yKauS1yWOSrM"},"source":["One can also group labels. For instance, *Electrical* has the following groups (run code below). \n","\n","#### Question 3, Data Manipulation\n","Why would one want to group labels of, for instance, Electrical? Would you advise doing this to improve the machine learning model? Why not or if so which and why?"]},{"cell_type":"code","metadata":{"id":"mMoSFySjOq9A"},"source":["name = \"Electrical\"\n","print(data[name].value_counts())\n","fig,ax = plt.subplots(1,2,figsize=(13,5))\n","sns.countplot(x=data[name], ax=ax[0]);\n","sns.boxplot(x=name, y='SalePrice',data=data, ax=ax[1]);"],"execution_count":null,"outputs":[]},{"cell_type":"markdown","metadata":{"id":"l4bPy2THSlGg"},"source":["The following code groups the non-standard circuits. After running the line below, one can run the code above and see if it worked. If one would not want to group them, the small groups should be removed, as they are just too infrequent for the machine learning model to see a pattern."]},{"cell_type":"markdown","metadata":{"id":"vhvmwB6l-pFV"},"source":["Some categorical features have some order in them, as they represent qualities, for instance, where one class is better than the other. We will represent this in the model by translating these variables into numeric values. These are called ordinal variables."]},{"cell_type":"code","metadata":{"id":"wZBN67hZ-pfg"},"source":["# Encode ordinal features as numeric\n","data = data.replace({'BsmtCond':{'None':0, 'Po':1, 'Fa':2, 'TA':3, 'Gd':4, 'Ex':5},\n","                     'BsmtExposure':{'None':0, 'Mn':1, 'Av': 2, 'Gd':3},\n","                     'BsmtFinType1':{'None':0, 'Unf':1, 'LwQ': 2, 'Rec':3, 'BLQ':4, 'ALQ':5, 'GLQ':6},\n","                     'BsmtQual':{'None':0, 'Po':1, 'Fa':2, 'TA': 3, 'Gd':4, 'Ex':5},\n","                     'ExterCond':{'Po':1, 'Fa':2, 'TA': 3, 'Gd': 4, 'Ex':5},\n","                     'ExterQual':{'Po':1, 'Fa':2, 'TA': 3, 'Gd': 4, 'Ex':5},\n","                     'FireplaceQu':{'None':0, 'Po':1, 'Fa':2, 'TA':3, 'Gd':4, 'Ex':5},\n","                     'HeatingQC':{'Po':1, 'Fa':2, 'TA':3, 'Gd':4, 'Ex':5},\n","                     'KitchenQual':{'Po':1, 'Fa':2, 'TA':3, 'Gd':4, 'Ex':5},\n","                     'LotShape':{'IR3':1, 'IR2':2, 'IR1':3, 'Reg':4},\n","                     'PavedDrive':{'N':0, 'P':1, 'Y':2}})"],"execution_count":null,"outputs":[]},{"cell_type":"code","metadata":{"id":"bp5bfhFYPP9t"},"source":["data = data.replace({\"Electrical\" : {\"SBrkr\" : \"SBrkr\", \"FuseA\" : \"Fuse\", \"FuseF\" : \"Fuse\", \"FuseP\" : \"Fuse\"}})"],"execution_count":null,"outputs":[]},{"cell_type":"markdown","metadata":{"id":"UMLMY3SYX3Fa"},"source":["Next, we are looking into correlations between variables. For instance, comparing Yearbuilt and GarageYrBlt. One can also here try different variables by replacing the names between the quotes.\n","\n","#### Question 4, Correlations in Time\n","Can you explain the strange pattern in the next figure? Do you think it is necessary to keep both variables or would only keeping one be enough?"]},{"cell_type":"code","metadata":{"id":"byEVjnL3iRu0"},"source":["sns.scatterplot(x=data['YearBuilt'], y=data['GarageYrBlt']);"],"execution_count":null,"outputs":[]},{"cell_type":"markdown","metadata":{"id":"cAJYth-198fc"},"source":["#### Question 5, Correlations in Space\n","Taking it even further, there is also a clear relationship between the number of garage car spots and the garage area, would you keep them both?"]},{"cell_type":"code","metadata":{"id":"CChk3gk2fByO"},"source":["sns.regplot(x=data['GarageCars'], y=data['GarageArea']);"],"execution_count":null,"outputs":[]},{"cell_type":"markdown","metadata":{"id":"IvPwD0kLDIa-"},"source":["## Machine Learning\n","First, we need to determine which variables are taken into account by removing or adding variable names to the list below. It is initialized to all variables. Next, the code splits the training set into a training and test set. "]},{"cell_type":"code","metadata":{"id":"usgxAxcPDIDP"},"source":["features = ['GrLivArea','TotalBsmtSF','1stFlrSF','2ndFlrSF',\n","'Bedroom','TotRmsAbvGrd',\n","'Neighborhood','MSZoning',\n","'OverallQual','OverallCond',\n","'YearBuilt','YearRemodAdd',\n","'MSSubClass','BldgType','HouseStyle',\n","'RoofStyle',\n","'Exterior1st','Exterior2nd','ExterQual','ExterCond',\n","'MasVnrType','MasVnrArea',\n","'Foundation',\n","'BsmtQual','BsmtCond','BsmtExposure','BsmtFinType1',\n","'BsmtFinSF1','BsmtUnfSF',\n","'CentralAir','Electrical','HeatingQC',\n","'BsmtFullBath','BsmtHalfBath',\n","'FullBath','HalfBath',\n","'Kitchen','KitchenQual',\n","'Fireplaces','FireplaceQu',\n","'GarageType','GarageYrBlt','GarageFinish','GarageCars','GarageArea',\n","'Fence','WoodDeckSF','OpenPorchSF','EnclosedPorch','ScreenPorch',\n","'LotFrontage','LotArea','LotShape','LotConfig',\n","'PavedDrive']\n","\n","X = data.dropna(axis=0)\n","y = X['SalePrice']\n","X = X[features]\n","X = pd.get_dummies(X)\n","X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.30, random_state=42)"],"execution_count":null,"outputs":[]},{"cell_type":"markdown","metadata":{"id":"iBNrz2BvAQji"},"source":["### Decision Tree Regression \n","Now, let's run the first model, one decision tree. The decision tree is a simple machine learning model for getting started with regression tasks. Each internal (non-leaf) node denotes a test on an attribute, each branch represents the outcome of a test, and each leaf (or terminal) node holds a class label that is predicted.\n","\n","The model outputs the Mean Absolute Error, the Mean Squared Error, and the Root Mean Squared Error, these need to be as low as possible. Next, a figure shows the predicted values versus the actual values. And lastly, it displays the 15 most important features for the model."]},{"cell_type":"code","metadata":{"id":"pFIbsG_9Bx-4"},"source":["dtreg = DecisionTreeRegressor(random_state = 42)\n","dtreg.fit(X_train, y_train)\n","\n","dtr_pred = dtreg.predict(X_test)\n","dtr_pred= dtr_pred.reshape(-1,1)\n","print('MAE:', metrics.mean_absolute_error(y_test, dtr_pred))\n","print('MSE:', metrics.mean_squared_error(y_test, dtr_pred))\n","print('RMSE:', np.sqrt(metrics.mean_squared_error(y_test, dtr_pred)))\n","\n","plt.figure(figsize=(15,8))\n","plt.scatter(y_test,dtr_pred,c='green')\n","plt.xlabel('Actual price')\n","plt.ylabel('Predicted price')\n","plt.show();\n","\n","# Feature Importance\n","feats = {}\n","for feature, importance in zip(X.columns, dtreg.feature_importances_):\n","    feats[feature] = importance\n","importances = pd.DataFrame.from_dict(feats, orient='index').rename(columns={0: 'Gini-importance'})\n","importances.sort_values(by='Gini-importance',ascending=False).head(15)"],"execution_count":null,"outputs":[]},{"cell_type":"markdown","metadata":{"id":"X-fkuU71KxBV"},"source":["### Random Forest\n","A Random Forest is an ensemble technique using  multiple decision trees and a technique called Bootstrap Aggregation, commonly known as bagging. This involves training each decision tree on a different data sample where sampling is done with replacement. The final prediction is the average of the prediction of every decision tree.\n","\n","### Question 6, Results\n","Which model is better, the Random Forest or the Decision tree, and why?\n","\n","### Question 7, Parameter Tuning\n","One can try to improve the Random Forest by changing the number at *n_estimators* on the first line (this represents the number of decision trees in the forest). Does adding more trees improve the prediction?"]},{"cell_type":"code","metadata":{"id":"3pMgarP4E2uU"},"source":["rfr = RandomForestRegressor(n_estimators = 30, random_state = 42)\n","rfr.fit(X_train, y_train)\n","\n","rfr_pred = rfr.predict(X_test)\n","rfr_pred= rfr_pred.reshape(-1,1)\n","print('MAE:', metrics.mean_absolute_error(y_test, rfr_pred))\n","print('MSE:', metrics.mean_squared_error(y_test, rfr_pred))\n","print('RMSE:', np.sqrt(metrics.mean_squared_error(y_test, rfr_pred)))\n","\n","plt.figure(figsize=(15,8))\n","plt.scatter(y_test,rfr_pred,c='green')\n","plt.xlabel('Actual price')\n","plt.ylabel('Predicted price')\n","plt.show();\n","\n","# Feature Importance\n","feats = {}\n","for feature, importance in zip(X.columns, rfr.feature_importances_):\n","    feats[feature] = importance\n","importances = pd.DataFrame.from_dict(feats, orient='index').rename(columns={0: 'Gini-importance'})\n","importances.sort_values(by='Gini-importance',ascending=False).head(15)"],"execution_count":null,"outputs":[]},{"cell_type":"markdown","metadata":{"id":"KPfQNj_9NM6I"},"source":["### Quention 8, Feature Selection\n","Feel free to try to improve performance by removing or adding features/variables to the model. One can for instance start with an empty model and build it up with reasonable features till performance does not increase anymore. In the cell just below the header **Machine Learning** one can determine which features are taken into account. Does leaving out features that are correlated with each other improve performance? "]},{"cell_type":"markdown","metadata":{"id":"8-M6HMT1_1CS"},"source":["If time allows, you can look at the following parts.\n","\n","## Extra, Your Analyses\n","Next, you can analyze any other variable by yourself with the following lines of code. The first line just lists all variable names. The second performs an analysis of numerical variables. The third part of the code performs an analysis for nominal/categorical variables. Just put the name of the variable you want to analyze at the line *name = \"Name_of_Variable\"* between the quotes."]},{"cell_type":"code","metadata":{"id":"1L8riFSB_0RQ"},"source":["list(data.columns)"],"execution_count":null,"outputs":[]},{"cell_type":"code","metadata":{"id":"91ehl675ABhn"},"source":["# Nominal Analysis\n","name = \"Neighborhood\"\n","print(data[name].value_counts())\n","fig,ax = plt.subplots(1,2,figsize=(13,5))\n","chart1 = sns.countplot(x=data[name], ax=ax[0]);\n","chart1.set_xticklabels(chart1.get_xticklabels(), rotation=45);\n","chart2 = sns.boxplot(x=name, y='SalePrice',data=data, ax=ax[1]);\n","chart2.set_xticklabels(chart1.get_xticklabels(), rotation=45);\n","chart1;\n","chart2;"],"execution_count":null,"outputs":[]},{"cell_type":"code","metadata":{"id":"V7-A6wiMAB5m"},"source":["# Numerical Analysis\n","name = \"GrLivArea\"\n","print(data[name].describe())\n","sns.displot(data[name], kde=True);\n","sns.lmplot(x=name, y='SalePrice', data=data);"],"execution_count":null,"outputs":[]},{"cell_type":"markdown","metadata":{"id":"_dY3fyTYOJ2d"},"source":["### Question 9, Feature Engineering\n","One may think about new features. Can we create new features by adding or multiplying other features together? Or can we create new features by combining features in other ways? The best results are most often not obtained by the best machine learning model, but by the researcher that did the best feature engineering. "]},{"cell_type":"markdown","metadata":{"id":"DDrdi5GA3G7a"},"source":["## Extra, Missing Values\n","Almost every dataset contains missing values. A difficult question is what to do with them. Leave them as a separate group, remove the data points, or replacing the value with a logical other value?\n","\n","The next line shows how many missing values are still in the data set."]},{"cell_type":"code","metadata":{"id":"fip-_cdN3Fyc"},"source":["total = data.isnull().sum().sort_values(ascending=False)\n","percent = (data.isnull().sum() /\n","           data.isnull().count()).sort_values(ascending=False)\n","missing_data = pd.concat([total, percent], axis=1, keys=['Total', 'Percent'])\n","missing_data[missing_data.Total > 0]"],"execution_count":null,"outputs":[]},{"cell_type":"markdown","metadata":{"id":"oUJZaxVU4A07"},"source":["We have numeric variables (LotFrontage, GarageYrBlt, MasVnrArea) and categorical (MasVnrType, Electrical) with missing values. \n","\n","#### Question 10, Data Cleaning\n","How would you replace missing values of these variables or would you remove the corresponding data point?"]}]}